-- Cette ligne permet d'afficher des traces dans la console pendant l'éxécution
io.stdout:setvbuf('no')

-- Empèche Love de filtrer les contours des images quand elles sont redimentionnées
-- Indispensable pour du pixel art
love.graphics.setDefaultFilter("nearest")

-- Cette ligne permet de déboguer pas à pas dans ZeroBraneStudio
if arg[#arg] == "-debug" then require("mobdebug").start() end
--******************************************************
--*     Break Out 2017   Par Jean-Bernard (Bernie)     *
--******************************************************
-- Elements pour l'interface du menu 
ecranTitre = "menu"
texteTimer = 0
clignoteTexte = true

-- Elements pour le jeu 
local score
local vies 

-- La raquette
local raquette = {}
raquette.x = 0
raquette.y = 0
raquette.largeur = 150
raquette.hauteur = 20 

-- La balle
local balle = {}
balle.x = 0
balle.y = 0
balle.rayon = 10
balle.colle = false 
balle.vx = 0
balle.vy = 0

-- les briques 
local briques = {}
local niveau = {}


-- les sons 
-- Si on perd une vie 
local  soundPerdballe 
-- Si le balle touche la raquette
local soundBalleToucheRaquette
-- Si on detruit une brique 
local soundDetruitBrique
-- Si on a perdu  
local soundGameOver


-- Fonction pour initialiser le jeu 
function IniJeu()
  -- le score et la vies 
  score = 0 
  vies = 1 
  -- Chargement des fichiers sons du jeu 
  -- Son quand on perd une vie 
  soundPerdballe = love.audio.newSource("sons/Perdballe.wav","static")
  -- Son quand la balle touche la raquette
  soundBalleToucheRaquette = love.audio.newSource("sons/RaquetteTouche.wav","static")
  -- Son quand la balle détruit une brique 
  soundDetruitBrique = love.audio.newSource("sons/ColliBrique.wav","static")
  -- Son quand on a perdu  la partie 
  soundGameOver = love.audio.newSource("sons/GameOver.wav","static")
  -- Au lancement du jeu  la balle est collée sur la raquette
  balle.colle = true 
  
  -- Création des 15  briques 
  briques.hauteur = 25
  briques.largeur = largeur/15

  
  -- On initialise les briques à chaque nouvelle partie 
  niveau = {}
  local l,c
  for l = 1, 6 do
    niveau[l] = {}
    for c = 1,15 do
      niveau[l][c] = 1
    end  
  end  
  
end  



function love.load()
  largeur = love.graphics.getWidth()
  hauteur = love.graphics.getHeight()
  -- On intialise le jeu 
  IniJeu()
end



function love.update(dt)
  if ecranTitre == "jeu" then
    
    -- Gestion de la raquette
    raquette.y = hauteur - (raquette.hauteur/2)
    -- Deplacement de la raquette avec la souris 
    raquette.x = love.mouse.getX()
    -- Gestion de la balle 
    if balle.colle == true then
      balle.x = raquette.x 
      balle.y = raquette.y - (raquette.hauteur/2) - balle.rayon 
    else
      balle.x = balle.x + (balle.vx*dt)
      balle.y = balle.y + (balle.vy*dt) 
    end  
   
    -- Collision de la balle avec les briques 
    local c = math.floor (balle.x  / briques.largeur) + 1 --  On enlève la virgule 
    local l = math.floor (balle.y / briques. hauteur) + 1 
   
    if l >= 1 and l <= #niveau and c >= 1 and c <= 15 then
      if niveau[l][c] == 1 then
        balle.vy = 0 - balle.vy
        niveau[l][c] = 0
        -- The below 3 lines will reset the sound if it's still playing while the ball hits another brick.
        -- It could be done better, but it's fine like this for a start.
        if soundDetruitBrique:isPlaying() then
          soundDetruitBrique:stop()
        end
        soundDetruitBrique : play()
        score = score + 10 
      end 
     end 
      
    -- Collision de la balle avec les murs 
    -- Mur de droite
    if balle.x > largeur then
      balle.vx = 0 - balle.vx
      balle.x = largeur
    end 
    -- Mur de gauche 
    if balle.x < 0 then
      balle.vx = 0 - balle.vx
      balle.x = 0
    end
    -- Mur du haut 
    if balle.y < 0  then
      balle.vy = 0 - balle.vy
      balle.y = 0
    end  
    
    -- Si la balle sort en bas on perd une vie et on repositionne la balle sur la raquette
    if balle.y > hauteur then
      -- On joue un son quand la balle sort 
      soundPerdballe:play()
      vies = vies -1
      balle.colle = true 
    end   
  
    -- Collision de la balle avec la raquette 
    local collisionRaquette = raquette.y - (raquette.hauteur/2) - balle.rayon 
    if balle.y > collisionRaquette then
      -- On joue un son quand la balle touche la raquette
      soundBalleToucheRaquette:play()
      local distance = math.abs(raquette.x - balle.x)
      if distance < raquette.largeur/2 then
        balle.vy = 0 - balle.vy
        balle.y = collisionRaquette
      end  
    end 
    -- si on a plus de vie on perd la partie et on affiche l'écran  game over 
    if vies == 0 then
      ecranTitre = "gameover"
    end
  end  
end



-- I moved the below two functions out of DrawMenu, because it was wasteful.

 -- Fonction pour dessiner le  jeu 
function DrawJeu()
  -- Dessin des briques qui seront de couleurs rouges 
  local l,c
  local bx,by = 0,0
  for l =1,6 do
    bx = 0 
    for c= 1,15 do
      if niveau[l][c] == 1 then
        love.graphics.setColor(255,0,0) 
        love.graphics.rectangle("fill", bx + 1,by + 1,briques.largeur - 2,briques.hauteur - 2)
      end 
      bx = bx + briques.largeur
    end 
    by = by + briques.hauteur
  end

  -- Affiche la couleur violette dans la fenetre du jeu
  love.graphics.setBackgroundColor(64,0,128)
  -- Affichage des textes score et vies de taille de 25 
  -- le score 
  love.graphics.setNewFont(25)
  love.graphics.setColor(255,255,255)
  love.graphics.print("Score : "..score,10,0)
  -- La vie  
  love.graphics.setColor(255,255,255)
  love.graphics.print("Vies : "..vies,630,0)
  
  -- Dessin de la raquette de couleur bleue
  love.graphics.setColor(0,128,192)
  love.graphics.rectangle("fill",raquette.x - (raquette.largeur/2),raquette.y - (raquette.hauteur/2),raquette.largeur,raquette.hauteur)
  -- Dessin de la balle de couleur jaune 
  love.graphics.setColor(255,255,0)
  love.graphics.circle("fill",balle.x,balle.y,balle.rayon)

end -- You were missing this end before... or rather, it was misplaced.



function DrawGameOver()
  soundGameOver : play() -- This will repeat infinitely like this...
  -- Affiche le titre  du jeu qui aura une couleur 
  love.graphics.setNewFont(100)
  love.graphics.setColor(255,255,255)
  love.graphics.printf("GAME OVER",125,200,880,"center")
  --Affiche le deuxieme texte qui aura une couleur 
  love.graphics.setNewFont(50)
  love.graphics.setColor(255,255,255)
  love.graphics.print("Press R",250,480)
end 



-- Fonction pour dessiner le menu
function DrawMenu()
  -- Affiche le titre  du jeu qui aura une couleur bleue  avec un contour blanc 
  love.graphics.setNewFont(100)
  love.graphics.setColor(255,255,255) -- contour blanc 
  love.graphics.printf("BREAK OUT",120,200,580,"center")
  love.graphics.setColor(0,128,255) -- couleur bleue 
  love.graphics.printf("BREAK OUT",125,200,580,"center")
  -- Affiche le texte press space qui clignote de couleur blanc 
  texteTimer = texteTimer + 1
  if texteTimer % 40 == 0 then
    if clignoteTexte == false then
      clignoteTexte = true
    else
      clignoteTexte = false 
    end 
  end 
  if clignoteTexte == true then
    love.graphics.setNewFont(50)
    love.graphics.setColor(255,255,255) 
    love.graphics.print("Press Space",250,480)
  end


end


    
function love.draw()
  if ecranTitre == "menu" then
    DrawMenu()
  elseif ecranTitre == "jeu" then 
    DrawJeu()
  elseif ecranTitre == "gameover" then
    DrawGameOver()
  end
end


-- Si on clique avec le bouton de la souris on lance la balle 
function love.mousepressed(x,x,n)
  -- Only do this if we're in the game; otherwise the ball might start at the top left corner...
  if ecranTitre == "jeu" and balle.colle == true then
    balle.colle = false 
    balle.vx = 200
    balle.vy = -200
  end  
end  



function love.keypressed(key)
  -- Si on appuye sur la barre d'espace on lancement le jeu
  if ecranTitre == "menu" then
    if key == "space" then
      ecranTitre = "jeu"
    end
  end

  -- Si on appuye sur la touche r on relance le jeu 
  if ecranTitre == "gameover" then
    if key == "r" then
      -- Maybe you should re-initialize / restart the game here...
      IniJeu()
      ecranTitre = "menu"
    end  
  end  
end